@@ -12,7 +12,7 @@
 #include "./wicdec.h"
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "webp/config.h"
 #endif
 
 #include <stdio.h>
